#ifndef __RUBY_H__
#define __RUBY_H__

#include "Canvas.h"
#include "MainWnd.h"
#include "TextBox.h"
#include <ruby.h>
#include <window/Window.h>
#include <map>
#include <vector>

class Ruby{
public:
	Ruby();
	~Ruby();

	bool Load(const TCHAR *file);

	bool Event_OnRButtonDown(int x,int y,bool isDClick);
	bool Event_OnLButtonDown(int x,int y,bool isDClick);
	bool Event_OnLButtonUp(int x,int y);
	bool Event_OnMouseMove(int x,int y);
	bool Event_OnKeyDown(UINT key);
	bool Event_OnKeyUp(UINT key);

	bool Event_TextBox_OnChar(VALUE handler,TCHAR ch);
	bool Event_TextBox_OnKeyDown(VALUE handler,UINT key);
	bool Event_TextBox_OnKeyUp(VALUE handler,UINT key);

private:
	void RubyInit();
	ablib::string GetErrorInfo();
	static VALUE sGetErrorInfo(VALUE module);

	VALUE m_Deskdrawmodule;
	
	static VALUE Main_SwitchMode(VALUE self,VALUE mode);
	static VALUE Main_GetMode(VALUE self);
	static VALUE Main_SetEventHandler(VALUE self,VALUE klass);
	static VALUE Main_Quit(VALUE selft);
	static VALUE Main_IsKeyDown(VALUE self,VALUE key);
	static VALUE Main_Invalidate();

	VALUE m_eventhandler;

	typedef VALUE (*RUBY_FUNC)(...);

	// MenuNX
	struct Ruby_Menu_Impl{
		Ruby_Menu_Impl(ablib::window::Wnd *wnd) : menu(wnd){m_str2int[_T("")] = 0;m_int2str.push_back(_T(""));}
		ablib::window::Menu menu;
		std::map<ablib::string,int> m_str2int;
		std::vector<ablib::string> m_int2str;
	};
	struct Ruby_Menu{
		Ruby_Menu_Impl *menu_impl;
	};
	static VALUE Menu_new(VALUE self);
	static void Menu_free(Ruby_Menu *self);
	static VALUE Menu_Add(VALUE self,VALUE id,VALUE parent,VALUE str);
	static VALUE Menu_CreatePopupMenu(VALUE self,VALUE x,VALUE y);
	static VALUE Menu_Clear(VALUE self);

	// CanvasNX
	struct Ruby_Canvas{
		Ruby_Canvas() : canvas(NULL){}
		Canvas *canvas;
	};
	static VALUE Canvas_new(VALUE self);
	static void Canvas_free(Ruby_Canvas *self);
	static VALUE Canvas_DrawLine(VALUE self,VALUE left,VALUE top,VALUE right,VALUE bottom);
	static VALUE Canvas_WriteText(VALUE self,VALUE x,VALUE y,VALUE str);
	static VALUE Canvas_FillRect(VALUE self,VALUE left,VALUE top,VALUE right,VALUE bottom);
	static VALUE Canvas_Undo(VALUE self);
	static VALUE Canvas_Redo(VALUE self);
	static VALUE Canvas_AddUndoChain(VALUE self);
	static VALUE Canvas_SetWindow(VALUE self);
	static VALUE Canvas_SetLineStyle(VALUE self,VALUE red,VALUE green,VALUE blue,VALUE width,VALUE style);
	static VALUE Canvas_SetFont(VALUE self,VALUE name,VALUE size,VALUE red,VALUE green,VALUE blue);
	static VALUE Canvas_SetFillColor(VALUE self,VALUE red,VALUE green,VALUE blue);
	static VALUE Canvas_GetWidth(VALUE self);
	static VALUE Canvas_GetHeight(VALUE self);

	// TextBoxNX
	struct Ruby_TextBox{
		Ruby_TextBox() : textbox(NULL){}
		TextBox *textbox;
	};
	static VALUE TextBox_new(VALUE self,VALUE left,VALUE top,VALUE right,VALUE bottom);
	static void TextBox_free(Ruby_TextBox *self);
	static void TextBox_mark(Ruby_TextBox *self);
	static VALUE TextBox_IsCreated(VALUE self);
	static VALUE TextBox_SetText(VALUE self,VALUE str);
	static VALUE TextBox_GetText(VALUE self);
	static VALUE TextBox_Destroy(VALUE self);
	static VALUE TextBox_SetSize(VALUE self,VALUE left,VALUE top,VALUE right,VALUE bottom);
	static VALUE TextBox_SetEventHandler(VALUE self,VALUE handler);
	struct ClassOnCharArg{VALUE handler;VALUE key;};
	static VALUE sEvent_TextBox_OnChar(VALUE val);
	struct ClassOnKeyArg{VALUE handler;VALUE key;};
	static VALUE sEvent_TextBox_OnKeyDown(VALUE val);
	static VALUE sEvent_TextBox_OnKeyUp(VALUE val);

	// Cxg֘A
	struct OnMouseArg{VALUE x;VALUE y;VALUE isDClick;};
	static VALUE sEvent_OnRButtonDown(VALUE val);
	static VALUE sEvent_OnLButtonDown(VALUE val);
	static VALUE sEvent_OnLButtonUp(VALUE val);
	static VALUE sEvent_OnMouseMove(VALUE val);

	static VALUE sEvent_OnKeyDown(VALUE val);

	static VALUE sEvent_OnKeyUp(VALUE val);




};

#endif //__RUBY_H__

